import stripe
from stripe.test.helper import (
    StripeResourceTest, DUMMY_PLAN
)


class PlanTest(StripeResourceTest):

    def test_create_plan(self):
        stripe.Plan.create(**DUMMY_PLAN)

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/plans',
            DUMMY_PLAN,
            None
        )

    def test_delete_plan(self):
        p = stripe.Plan(id="pl_delete")
        p.delete()

        self.requestor_mock.request.assert_called_with(
            'delete',
            '/v1/plans/pl_delete',
            {},
            None
        )

    def test_update_plan(self):
        p = stripe.Plan(id="pl_update")
        p.name = "Plan Name"
        p.save()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/plans/pl_update',
            {
                'name': 'Plan Name',
            },
            None
        )



import stripe
from stripe.test.helper import StripeResourceTest


class PayoutTest(StripeResourceTest):

    def test_list_payouts(self):
        stripe.Payout.list()
        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/payouts',
            {}
        )

    def test_cancel_payout(self):
        payout = stripe.Payout(id='po_cancel')
        payout.cancel()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/payouts/po_cancel/cancel',
            {},
            None
        )



import stripe
from stripe.test.helper import StripeApiTestCase


class ListObjectTests(StripeApiTestCase):

    def setUp(self):
        super(ListObjectTests, self).setUp()

        self.lo = stripe.resource.ListObject.construct_from({
            'id': 'me',
            'url': '/my/path',
            'data': ['foo'],
        }, 'mykey')

        self.mock_response([{
            'object': 'charge',
            'foo': 'bar',
        }])

    def assertResponse(self, res):
        self.assertTrue(isinstance(res[0], stripe.Charge))
        self.assertEqual('bar', res[0].foo)

    def test_for_loop(self):
        seen = []

        for item in self.lo:
            seen.append(item)

        self.assertEqual(['foo'], seen)

    def test_list(self):
        res = self.lo.list(myparam='you')

        self.requestor_mock.request.assert_called_with(
            'get', '/my/path', {'myparam': 'you'}, None)

        self.assertResponse(res)

    def test_create(self):
        res = self.lo.create(myparam='eter')

        self.requestor_mock.request.assert_called_with(
            'post', '/my/path', {'myparam': 'eter'}, None)

        self.assertResponse(res)

    def test_retrieve(self):
        res = self.lo.retrieve('myid', myparam='cow')

        self.requestor_mock.request.assert_called_with(
            'get', '/my/path/myid', {'myparam': 'cow'}, None)

        self.assertResponse(res)


class AutoPagingTests(StripeApiTestCase):

    @staticmethod
    def pageable_model_response(ids, has_more):
        return {
            'object': 'list',
            'url': '/v1/pageablemodels',
            'data': [{'id': id, 'object': 'pageablemodel'} for id in ids],
            'has_more': has_more,
        }

    def test_iter_one_page(self):
        lo = stripe.resource.ListObject.construct_from(
            self.pageable_model_response(['pm_123', 'pm_124'], False),
            'mykey'
        )

        self.requestor_mock.request.assert_not_called()

        seen = [item['id'] for item in lo.auto_paging_iter()]

        self.assertEqual(['pm_123', 'pm_124'], seen)

    def test_iter_two_pages(self):
        lo = stripe.resource.ListObject.construct_from(
            self.pageable_model_response(['pm_123', 'pm_124'], True),
            'mykey'
        )
        lo._retrieve_params = {'foo': 'bar'}

        self.mock_response(
            self.pageable_model_response(['pm_125', 'pm_126'], False)
        )

        seen = [item['id'] for item in lo.auto_paging_iter()]

        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/pageablemodels',
            {
                'starting_after': 'pm_124',
                'foo': 'bar'
            },
            None
        )

        self.assertEqual(['pm_123', 'pm_124', 'pm_125', 'pm_126'], seen)

    def test_class_method_two_pages(self):
        self.mock_response({
            'object': 'list',
            'data': [{'id': 'ch_001'}],
            'url': '/v1/charges',
            'has_more': False,
        })

        seen = [item['id'] for item in stripe.Charge.auto_paging_iter(
            limit=25,
            foo='bar'
        )]

        self.requestor_mock.request.assert_called_with(
            'get', '/v1/charges', {'limit': 25, 'foo': 'bar'}
        )

        self.assertEqual(['ch_001'], seen)



import stripe
from stripe.test.helper import StripeResourceTest


class ApplicationFeeTest(StripeResourceTest):

    def test_list_application_fees(self):
        stripe.ApplicationFee.list()
        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/application_fees',
            {}
        )


class ApplicationFeeRefundTest(StripeResourceTest):

    def test_fetch_refund(self):
        fee = stripe.ApplicationFee.construct_from({
            'id': 'fee_get_refund',
            'refunds': {
                'object': 'list',
                'url': '/v1/application_fees/fee_get_refund/refunds',
            }
        }, 'api_key')

        fee.refunds.retrieve("ref_get")

        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/application_fees/fee_get_refund/refunds/ref_get',
            {},
            None
        )

    def test_list_refunds(self):
        fee = stripe.ApplicationFee.construct_from({
            'id': 'fee_get_refund',
            'refunds': {
                'object': 'list',
                'url': '/v1/application_fees/fee_get_refund/refunds',
            }
        }, 'api_key')

        fee.refunds.list()

        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/application_fees/fee_get_refund/refunds',
            {},
            None
        )

    def test_update_refund(self):
        def side_effect(*args):
            raise stripe.InvalidRequestError('invalid', 'foo')

        self.requestor_mock.request.side_effect = side_effect

        refund = stripe.resource.ApplicationFeeRefund.construct_from({
            'id': "ref_update",
            'fee': "fee_update",
            'metadata': {},
        }, 'api_key')

        refund.metadata["key"] = "foo"

        try:
            refund.save()
        except stripe.InvalidRequestError:
            pass

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/application_fees/fee_update/refunds/ref_update',
            {
                'metadata': {
                    'key': 'foo',
                }
            },
            None
        )

        self.requestor_mock.request.side_effect = None

        refund.metadata["key"] = "bar"
        refund.save()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/application_fees/fee_update/refunds/ref_update',
            {
                'metadata': {
                    'key': 'bar',
                }
            },
            None
        )

    def test_modify_refund(self):
        stripe.resource.ApplicationFeeRefund.modify("fee_update", "ref_update",
                                                    metadata={'key': 'foo'},
                                                    api_key='api_key')
        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/application_fees/fee_update/refunds/ref_update',
            {
                'metadata': {
                    'key': 'foo',
                }
            },
            None
        )



import stripe
from stripe.test.helper import (
    StripeResourceTest, DUMMY_APPLE_PAY_DOMAIN
)


class ApplePayDomainTest(StripeResourceTest):

    def test_create_apple_pay_domain(self):
        stripe.ApplePayDomain.create(**DUMMY_APPLE_PAY_DOMAIN)
        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/apple_pay/domains',
            DUMMY_APPLE_PAY_DOMAIN,
            None
        )

    def test_retrieve_apple_pay_domain(self):
        stripe.ApplePayDomain.retrieve("apwc_test")
        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/apple_pay/domains/apwc_test',
            {},
            None
        )

    def test_list_apple_pay_domains(self):
        stripe.ApplePayDomain.list()
        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/apple_pay/domains',
            {},
        )

    def test_delete_apple_pay_domain(self):
        d = stripe.ApplePayDomain(id='apwc_delete')
        d.delete()

        self.requestor_mock.request.assert_called_with(
            'delete',
            '/v1/apple_pay/domains/apwc_delete',
            {},
            None
        )



import datetime
import time
import warnings

import stripe
from stripe.test.helper import (
    StripeResourceTest, DUMMY_CARD, DUMMY_PLAN, NOW
)


class CustomerTest(StripeResourceTest):

    def test_list_customers(self):
        stripe.Customer.list()
        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/customers',
            {},
        )

    def test_create_customer(self):
        stripe.Customer.create(description="foo bar", card=DUMMY_CARD,
                               coupon='cu_discount', idempotency_key='foo')
        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers',
            {
                'coupon': 'cu_discount',
                'description': 'foo bar',
                'card': DUMMY_CARD
            },
            {'Idempotency-Key': 'foo'}
        )

    def test_unset_description(self):
        customer = stripe.Customer(id="cus_unset_desc")
        customer.description = "Hey"
        customer.save(idempotency_key='foo')

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers/cus_unset_desc',
            {
                'description': 'Hey',
            },
            {'Idempotency-Key': 'foo'}
        )

    def test_del_coupon(self):
        customer = stripe.Customer(id="cus_unset_desc")
        customer.description = "bar"
        customer.coupon = "foo"
        del customer.coupon
        customer.save()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers/cus_unset_desc',
            {
                'description': 'bar'
            },
            None
        )

    def test_cannot_set_empty_string(self):
        customer = stripe.Customer()
        self.assertRaises(ValueError, setattr, customer, "description", "")

    def test_customer_add_card(self):
        customer = stripe.Customer.construct_from({
            'id': 'cus_add_card',
            'sources': {
                'object': 'list',
                'url': '/v1/customers/cus_add_card/sources',
            },
        }, 'api_key')
        customer.sources.create(card=DUMMY_CARD, idempotency_key='foo')

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers/cus_add_card/sources',
            {
                'card': DUMMY_CARD,
            },
            {'Idempotency-Key': 'foo'}
        )

    def test_customer_add_source(self):
        customer = stripe.Customer.construct_from({
            'id': 'cus_add_source',
            'sources': {
                'object': 'list',
                'url': '/v1/customers/cus_add_source/sources',
            },
        }, 'api_key')
        customer.sources.create(source=DUMMY_CARD, idempotency_key='foo')

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers/cus_add_source/sources',
            {
                'source': DUMMY_CARD,
            },
            {'Idempotency-Key': 'foo'}
        )

    def test_customer_update_card(self):
        card = stripe.Card.construct_from({
            'customer': 'cus_update_card',
            'id': 'ca_update_card',
        }, 'api_key')
        card.name = 'The Best'
        card.save(idempotency_key='foo')

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers/cus_update_card/sources/ca_update_card',
            {
                'name': 'The Best',
            },
            {'Idempotency-Key': 'foo'}
        )

    def test_customer_update_source(self):
        source = stripe.BitcoinReceiver.construct_from({
            'customer': 'cus_update_source',
            'id': 'btcrcv_update_source',
        }, 'api_key')
        source.name = 'The Best'
        source.save(idempotency_key='foo')

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers/cus_update_source/sources/btcrcv_update_source',
            {
                'name': 'The Best',
            },
            {'Idempotency-Key': 'foo'}
        )

    def test_customer_update_alipay_account(self):
        aa = stripe.AlipayAccount.construct_from({
            'customer': 'cus_update_alipay',
            'id': 'ali_update',
        }, 'api_key')
        aa.metadata = {'name': 'The Best'}
        aa.save(idempotency_key='foo')

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers/cus_update_alipay/sources/ali_update',
            {
                'metadata': {'name': 'The Best'},
            },
            {'Idempotency-Key': 'foo'}
        )

    def test_customer_delete_card(self):
        card = stripe.Card.construct_from({
            'customer': 'cus_delete_card',
            'id': 'ca_delete_card',
        }, 'api_key')
        card.delete()

        self.requestor_mock.request.assert_called_with(
            'delete',
            '/v1/customers/cus_delete_card/sources/ca_delete_card',
            {},
            None
        )

    def test_customer_delete_source(self):
        source = stripe.BitcoinReceiver.construct_from({
            'customer': 'cus_delete_source',
            'id': 'btcrcv_delete_source',
        }, 'api_key')
        source.delete()

        self.requestor_mock.request.assert_called_with(
            'delete',
            '/v1/customers/cus_delete_source/sources/btcrcv_delete_source',
            {},
            None
        )

    def test_customer_delete_alipay_account(self):
        aa = stripe.AlipayAccount.construct_from({
            'customer': 'cus_delete_alipay',
            'id': 'ali_delete',
        }, 'api_key')
        aa.delete()

        self.requestor_mock.request.assert_called_with(
            'delete',
            '/v1/customers/cus_delete_alipay/sources/ali_delete',
            {},
            None
        )

    def test_customer_delete_bank_account(self):
        source = stripe.BankAccount.construct_from({
            'customer': 'cus_delete_source',
            'id': 'ba_delete_source',
        }, 'api_key')
        source.delete()

        self.requestor_mock.request.assert_called_with(
            'delete',
            '/v1/customers/cus_delete_source/sources/ba_delete_source',
            {},
            None
        )

    def test_customer_verify_bank_account(self):
        source = stripe.BankAccount.construct_from({
            'customer': 'cus_verify_source',
            'id': 'ba_verify_source',
        }, 'api_key')
        source.verify()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers/cus_verify_source/sources/ba_verify_source/verify',
            {},
            None
        )

    def test_customer_modify_source(self):
        stripe.Customer.modify_source(
            'cus_test', 'card_test',
            exp_month=NOW.month, exp_year=NOW.year + 1)

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers/cus_test/sources/card_test',
            {
                'exp_month': NOW.month,
                'exp_year': NOW.year + 1,
            },
            None
        )


class CustomerPlanTest(StripeResourceTest):

    def test_create_customer(self):
        stripe.Customer.create(plan=DUMMY_PLAN['id'], card=DUMMY_CARD)

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers',
            {
                'card': DUMMY_CARD,
                'plan': DUMMY_PLAN['id'],
            },
            None
        )

    def test_legacy_update_subscription(self):
        with warnings.catch_warnings(record=True) as w:
            customer = stripe.Customer(id="cus_legacy_sub_update")
            customer.update_subscription(idempotency_key='foo',
                                         plan=DUMMY_PLAN['id'])

            self.requestor_mock.request.assert_called_with(
                'post',
                '/v1/customers/cus_legacy_sub_update/subscription',
                {
                    'plan': DUMMY_PLAN['id'],
                },
                {'Idempotency-Key': 'foo'}
            )

            self.assertEqual(len(w), 1)
            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))

    def test_legacy_delete_subscription(self):
        with warnings.catch_warnings(record=True) as w:
            customer = stripe.Customer(id="cus_legacy_sub_delete")
            customer.cancel_subscription()

            self.requestor_mock.request.assert_called_with(
                'delete',
                '/v1/customers/cus_legacy_sub_delete/subscription',
                {},
                None
            )

            self.assertEqual(len(w), 1)
            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))

    def test_list_customer_subscriptions(self):
        customer = stripe.Customer.construct_from({
            'id': 'cus_foo',
            'subscriptions': {
                'object': 'list',
                'url': 'v1/customers/cus_foo/subscriptions',
            }
        }, 'api_key')

        customer.subscriptions.all()

        self.requestor_mock.request.assert_called_with(
            'get',
            'v1/customers/cus_foo/subscriptions',
            {},
            None
        )

    def test_create_customer_subscription(self):
        customer = stripe.Customer.construct_from({
            'id': 'cus_sub_create',
            'subscriptions': {
                'object': 'list',
                'url': '/v1/customers/cus_sub_create/subscriptions',
            }
        }, 'api_key')

        customer.subscriptions.create(plan=DUMMY_PLAN['id'], coupon='foo')

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/customers/cus_sub_create/subscriptions',
            {
                'plan': DUMMY_PLAN['id'],
                'coupon': 'foo',
            },
            None
        )

    def test_retrieve_customer_subscription(self):
        customer = stripe.Customer.construct_from({
            'id': 'cus_foo',
            'subscriptions': {
                'object': 'list',
                'url': '/v1/customers/cus_foo/subscriptions',
            }
        }, 'api_key')

        customer.subscriptions.retrieve('sub_cus')

        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/customers/cus_foo/subscriptions/sub_cus',
            {},
            None
        )

    def test_update_customer_subscription(self):
        subscription = stripe.Subscription.construct_from({
            'id': "sub_update",
            'customer': "cus_foo",
        }, 'api_key')

        trial_end_dttm = datetime.datetime.now() + datetime.timedelta(days=15)
        trial_end_int = int(time.mktime(trial_end_dttm.timetuple()))

        subscription.trial_end = trial_end_int
        subscription.plan = DUMMY_PLAN['id']
        subscription.save()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/subscriptions/sub_update',
            {
                'plan': DUMMY_PLAN['id'],
                'trial_end': trial_end_int,
            },
            None
        )

    def test_delete_customer_subscription(self):
        subscription = stripe.Subscription.construct_from({
            'id': "sub_delete",
            'customer': "cus_foo",
        }, 'api_key')

        subscription.delete()

        self.requestor_mock.request.assert_called_with(
            'delete',
            '/v1/subscriptions/sub_delete',
            {},
            None
        )



from __future__ import print_function

import logging
import sys
import os
import re

import stripe


STRIPE_LOG = os.environ.get('STRIPE_LOG')

logger = logging.getLogger('stripe')

__all__ = [
    'StringIO',
    'parse_qsl',
    'json',
    'utf8',
    'log_info',
    'log_debug',
    'dashboard_link',
    'logfmt',
]

try:
    # When cStringIO is available
    import cStringIO as StringIO
except ImportError:
    import StringIO

try:
    from urlparse import parse_qsl
except ImportError:
    # Python < 2.6
    from cgi import parse_qsl

try:
    import json
except ImportError:
    json = None

if not (json and hasattr(json, 'loads')):
    try:
        import simplejson as json
    except ImportError:
        if not json:
            raise ImportError(
                "Stripe requires a JSON library, such as simplejson. "
                "HINT: Try installing the "
                "python simplejson library via 'pip install simplejson' or "
                "'easy_install simplejson', or contact support@stripe.com "
                "with questions.")
        else:
            raise ImportError(
                "Stripe requires a JSON library with the same interface as "
                "the Python 2.6 'json' library.  You appear to have a 'json' "
                "library with a different interface.  Please install "
                "the simplejson library.  HINT: Try installing the "
                "python simplejson library via 'pip install simplejson' "
                "or 'easy_install simplejson', or contact support@stripe.com"
                "with questions.")


def utf8(value):
    # Note the ordering of these conditionals: `unicode` isn't a symbol in
    # Python 3 so make sure to check version before trying to use it. Python
    # 2to3 will also boil out `unicode`.
    if sys.version_info < (3, 0) and isinstance(value, unicode):
        return value.encode('utf-8')
    else:
        return value


def is_appengine_dev():
    return ('APPENGINE_RUNTIME' in os.environ and
            'Dev' in os.environ.get('SERVER_SOFTWARE', ''))


def _console_log_level():
    if stripe.log in ['debug', 'info']:
        return stripe.log
    elif STRIPE_LOG in ['debug', 'info']:
        return STRIPE_LOG
    else:
        return None


def log_debug(message, **params):
    msg = logfmt(dict(message=message, **params))
    if _console_log_level() == 'debug':
        print(msg, file=sys.stderr)
    logger.debug(msg)


def log_info(message, **params):
    msg = logfmt(dict(message=message, **params))
    if _console_log_level() in ['debug', 'info']:
        print(msg, file=sys.stderr)
    logger.info(msg)


def _test_or_live_environment():
    if stripe.api_key is None:
        return
    match = re.match(r'sk_(live|test)_', stripe.api_key)
    if match is None:
        return
    return match.groups()[0]


def dashboard_link(request_id):
    return 'https://dashboard.stripe.com/{env}/logs/{reqid}'.format(
        env=_test_or_live_environment() or 'test',
        reqid=request_id,
    )


def logfmt(props):
    def fmt(key, val):
        # Check if val is already a string to avoid re-encoding into
        # ascii. Since the code is sent through 2to3, we can't just
        # use unicode(val, encoding='utf8') since it will be
        # translated incorrectly.
        if not isinstance(val, basestring):
            val = unicode(val)
        if re.search(r'\s', val):
            val = repr(val)
        # key should already be a string
        if re.search(r'\s', key):
            key = repr(key)
        return u'{key}={val}'.format(key=key, val=val)
    return u' '.join([fmt(key, val) for key, val in sorted(props.items())])



import stripe
from stripe.test.helper import (
    StripeApiTestCase, MyListable
)


class ListableAPIResourceTests(StripeApiTestCase):

    def test_all(self):
        self.mock_response({
            'object': 'list',
            'data': [
                {
                    'object': 'charge',
                    'name': 'jose',
                },
                {
                    'object': 'charge',
                    'name': 'curly',
                }
            ],
            'url': '/v1/charges',
            'has_more': False,
        })

        res = MyListable.list()

        self.requestor_mock.request.assert_called_with(
            'get', '/v1/mylistables', {})

        self.assertEqual(2, len(res.data))
        self.assertTrue(all(isinstance(obj, stripe.Charge)
                            for obj in res.data))
        self.assertEqual('jose', res.data[0].name)
        self.assertEqual('curly', res.data[1].name)



import os
import sys
import warnings

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

try:
    from distutils.command.build_py import build_py_2to3 as build_py
except ImportError:
    from distutils.command.build_py import build_py

path, script = os.path.split(sys.argv[0])
os.chdir(os.path.abspath(path))

install_requires = []

if sys.version_info < (2, 6):
    warnings.warn(
        'Python 2.5 is no longer officially supported by Stripe. '
        'If you have any questions, please file an issue on Github or '
        'contact us at support@stripe.com.',
        DeprecationWarning)
    install_requires.append('requests >= 0.8.8, < 0.10.1')
    install_requires.append('ssl')
else:
    install_requires.append('requests >= 0.8.8')


with open('LONG_DESCRIPTION.rst') as f:
    long_description = f.read()

# Don't import stripe module here, since deps may not be installed
sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'stripe'))
from version import VERSION

# Get simplejson if we don't already have json
if sys.version_info < (3, 0):
    try:
        from util import json
    except ImportError:
        install_requires.append('simplejson')


setup(
    name='stripe',
    cmdclass={'build_py': build_py},
    version=VERSION,
    description='Stripe python bindings',
    long_description=long_description,
    author='Stripe',
    author_email='support@stripe.com',
    url='https://github.com/stripe/stripe-python',
    packages=['stripe', 'stripe.test', 'stripe.test.resources'],
    package_data={'stripe': ['data/ca-certificates.crt']},
    install_requires=install_requires,
    test_suite='stripe.test.all',
    tests_require=['unittest2', 'mock'],
    use_2to3=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ])



import stripe
from stripe.test.helper import (
    StripeResourceTest, DUMMY_COUPON
)


class CouponTest(StripeResourceTest):

    def test_create_coupon(self):
        stripe.Coupon.create(**DUMMY_COUPON)
        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/coupons',
            DUMMY_COUPON,
            None
        )

    def test_update_coupon(self):
        coup = stripe.Coupon.construct_from({
            'id': 'cu_update',
            'metadata': {},
        }, 'api_key')
        coup.metadata["key"] = "value"
        coup.save()

        self.requestor_mock.request.assert_called_with(
            'post',
            "/v1/coupons/cu_update",
            {
                'metadata': {
                    'key': 'value',
                }
            },
            None
        )

    def test_delete_coupon(self):
        c = stripe.Coupon(id='cu_delete')
        c.delete()

        self.requestor_mock.request.assert_called_with(
            'delete',
            '/v1/coupons/cu_delete',
            {},
            None
        )

    def test_detach_coupon(self):
        customer = stripe.Customer(id="cus_delete_discount")
        customer.delete_discount()

        self.requestor_mock.request.assert_called_with(
            'delete',
            '/v1/customers/cus_delete_discount/discount',
        )



from __future__ import print_function

import sys
from collections import namedtuple

try:
    from unittest import mock
except ImportError:
    import mock

from stripe import util
from stripe.test.helper import StripeUnitTestCase

import __builtin__

PRINT_FUNC_STRING = __builtin__.__name__ + '.print'

LogTestCase = namedtuple('LogTestCase', 'env flag should_output')
FmtTestCase = namedtuple('FmtTestCase', 'props expected')


class UtilTests(StripeUnitTestCase):
    DUMMY_REQ_ID = 'req_qsxPhqHyLxcoaM'

    def test_test_apikey(self):
        with mock.patch('stripe.api_key', 'sk_test_KOWobxXidxNlIx'):
            link = util.dashboard_link(self.DUMMY_REQ_ID)
            self.assertEqual(
                link,
                'https://dashboard.stripe.com/test/logs/' + self.DUMMY_REQ_ID,
            )

    def test_live_apikey(self):
        with mock.patch('stripe.api_key', 'sk_live_axwITqZSgTUXSN'):
            link = util.dashboard_link(self.DUMMY_REQ_ID)
            self.assertEqual(
                link,
                'https://dashboard.stripe.com/live/logs/' + self.DUMMY_REQ_ID,
            )

    def test_no_apikey(self):
        with mock.patch('stripe.api_key', None):
            link = util.dashboard_link(self.DUMMY_REQ_ID)
            self.assertEqual(
                link,
                'https://dashboard.stripe.com/test/logs/' + self.DUMMY_REQ_ID,
            )

    def test_old_apikey(self):
        with mock.patch('stripe.api_key', 'axwITqZSgTUXSN'):
            link = util.dashboard_link(self.DUMMY_REQ_ID)
            self.assertEqual(
                link,
                'https://dashboard.stripe.com/test/logs/' + self.DUMMY_REQ_ID,
            )

    def patch_val(self, var, value):
        patcher = mock.patch(var, value)
        patcher.start()
        self.addCleanup(patcher.stop)

    def patch_mock(self, var):
        patcher = mock.patch(var)
        mock_val = patcher.start()
        self.addCleanup(patcher.stop)
        return mock_val

    def log_test_loop(self, test_cases, logging_func, logger_name):
        for case in test_cases:
            with self.subTest(env=case.env, flag=case.flag):
                try:
                    logger_mock = self.patch_mock(logger_name)
                    print_mock = self.patch_mock(PRINT_FUNC_STRING)
                    self.patch_val('stripe.log', case.flag)
                    self.patch_val('stripe.util.STRIPE_LOG', case.env)

                    logging_func('foo \nbar', y=3)  # function under test

                    if case.should_output:
                        print_mock.assert_called_once_with(
                            "message='foo \\nbar' y=3",
                            file=sys.stderr,
                        )
                    else:
                        print_mock.assert_not_called()
                    logger_mock.assert_called_once_with(
                        "message='foo \\nbar' y=3")
                finally:
                    self.doCleanups()

    def test_log_debug(self):
        # (STRIPE_LOG, stripe.log): should_output?
        test_cases = [
            LogTestCase(env=None, flag=None, should_output=False),
            LogTestCase(env=None, flag='debug', should_output=True),
            LogTestCase(env=None, flag='info', should_output=False),
            LogTestCase(env='debug', flag=None, should_output=True),
            LogTestCase(env='debug', flag='debug', should_output=True),
            LogTestCase(env='debug', flag='info', should_output=False),
            LogTestCase(env='info', flag=None, should_output=False),
            LogTestCase(env='info', flag='debug', should_output=True),
            LogTestCase(env='info', flag='info', should_output=False),
        ]
        self.log_test_loop(
            test_cases,
            logging_func=util.log_debug,
            logger_name='stripe.util.logger.debug',
        )

    def test_log_info(self):
        # (STRIPE_LOG, stripe.log): should_output?
        test_cases = [
            LogTestCase(env=None, flag=None, should_output=False),
            LogTestCase(env=None, flag='debug', should_output=True),
            LogTestCase(env=None, flag='info', should_output=True),
            LogTestCase(env='debug', flag=None, should_output=True),
            LogTestCase(env='debug', flag='debug', should_output=True),
            LogTestCase(env='debug', flag='info', should_output=True),
            LogTestCase(env='info', flag=None, should_output=True),
            LogTestCase(env='info', flag='debug', should_output=True),
            LogTestCase(env='info', flag='info', should_output=True),
        ]
        self.log_test_loop(
            test_cases,
            logging_func=util.log_info,
            logger_name='stripe.util.logger.info',
        )

    def test_logfmt(self):
        cases = [
            FmtTestCase(props={'foo': 'bar', 'message': 'yes'},
                        expected='foo=bar message=yes'),
            FmtTestCase(props={'foo': 'bar baz'},
                        expected="foo='bar baz'"),
            FmtTestCase(props={'b': 2, 'a': 1, 'c': 3},
                        expected='a=1 b=2 c=3'),
            FmtTestCase(props={'key with space': True},
                        expected="'key with space'=True"),
        ]
        for case in cases:
            with self.subTest(props=case.props):
                result = util.logfmt(case.props)
                self.assertEqual(result, case.expected)



import stripe
from stripe.test.helper import StripeResourceTest


class TransferTest(StripeResourceTest):

    def test_list_transfers(self):
        stripe.Transfer.list()
        self.requestor_mock.request.assert_called_with(
            'get',
            '/v1/transfers',
            {}
        )

    def test_cancel_transfer(self):
        transfer = stripe.Transfer(id='tr_cancel')
        transfer.cancel()

        self.requestor_mock.request.assert_called_with(
            'post',
            '/v1/transfers/tr_cancel/cancel',
            {},
            None
        )
